# simple config file reader using Python's syntax and parser


class ConfigParserError(Exception):
    pass


class ConfigParser:
    def __init__(self):
        self.args = {}       # config file arguments
        self.registered_args = {}

    def add_argument(self, name, type_=str, default=None):
        """Register argument name, type and default value."""
        self.registered_args[name] = {'type':type_, 'default':default}
        self.args[name] = default

    def load(self, fname):
        """Load config file and check loaded values."""
        self.parse(fname)
        self.check()

    def parse(self, fname):
        """Parse config file. Does not check values."""
        with open(fname) as f:
            exec(f.read(), self.args)

    def check(self):
        """Check loaded argument values against registered types."""
        for key in self.args.keys():
            if key == '__builtins__':
                continue
            if key in self.registered_args:
                # arg registered, check type
                type_ = self.registered_args[key]['type']
                if not isinstance(self.args[key], type_) and not self.args[key] is None:
                    raise ConfigParserError("Bad value of config parameter '%s': type is %s but should be %s"
                        % (key, type(self.args[key]), type_))
            else:
                raise ConfigParserError("Unknown config parameter '%s'." % key)

    def __getattr__(self, name):
        """Attribute access for config parameters.

        cfg.my_param <===> cfg.args['myparam']

        """
        if name in self.args:
            return self.args[name]
        else:
            raise AttributeError()

